@ECHO OFF
SETLOCAL ENABLEDELAYEDEXPANSION
SETLOCAL ENABLEEXTENSIONS

:MKIMG
:: Batch to create HD like images of arbitrary size and to format them with various geometries
:: by jaclaz Release 0.01 ALPHA 14 October 2007
:: This script is licensed under my careware license: 
:: http://home.graffiti.net/jaclaz:graffiti.net/careware.html
:: Usage:
:: MBRbatch.cmd target filesize[k|M|G]  Geometry PartType [/fsz] [/np]
:: by default the batch uses mksparse, if the additional parameter /fsz is used, fsz will be used instead of mksparse
:: if using fsz (Warning, unlike sparse files, a file made with fsz occupies the whole size on disk
:: syntax: fsz file size (in bytes, no limits - be careful)
:: It can be used to create a file or truncate/extend an existing one.
:: Example: fsz data.fil 15773
:: If using makesparse (Suggested on NTFS volumes):
:: syntax: mksparse <filename> [size[k|M|G]]
:: 1k=1,024 bytes
:: 1M=1,048,576 bytes (1,024x1,024)
:: 1G=1,073,741,824 (1,024x1,024x1,024)
:: to unify the way the two utilities work, the batch accepts for fsz the same suffixes mksparse uses
:: while fsz will accept any size, mksparse will only work with multiples of 512 
:: Please note that last optional parameter /np for NO PROMPTS suppresses ALL prompts for confirming deletion
:: of existing files and for formatting the newly created image
:: If any needed parameter is missing, batch will enter interactive mode.

SET Targetfile=%1
SET Targetname=%~n1
SET Filesize=%2
SET Tempvar1=%3
SET Tempvar2=%4
SET Progname=%5

:Begin
SET neededfiles=VDK.EXE VDK.SYS dsfi.exe fsz.exe MBRBATCH.CMD mksparse.exe
FOR %%A IN (%neededfiles%) DO IF NOT EXIST %%A ECHO Missing file: %%A& GOTO :ERROR1

IF NOT DEFINED Targetfile CALL :MKIMGI_TF
IF NOT DEFINED Filesize CALL :MKIMGI_FS
CALL :check_filesize
SET Geometries=255/63 128/63 16/63 64/32
IF NOT DEFINED Tempvar1 CALL :MKIMGI_G
IF NOT DEFINED Geometry FOR %%A in (%Geometries%) DO IF %Tempvar1%.==%%A. (SET Geometry=%%A)
IF NOT DEFINED Geometry CALL :MKIMGI_G



::We'd  better check for filesystems limits
CALL :Do_Divide %Filesize% 512
SET Totsectors=%Result%
SET PartTypes=01 04 06 07 0B 0C 0E
:: FAT12 (01) has a limit in 16,777,216 bytes, i.e. 4,096 x 4Kb clusters (8 sectors), i.e. 32,768 sectors 
IF %Totsectors% gtr 32768 SET PartTypes=04 06 07 0B 0C 0E
:: FAT16 <32 Mb  (04) has a limit in 33,554,432 bytes, i.e. 16,384 x 2 Kb clusters (4 sectors), i.e. 65,536 sectors 
:: to be more exact this FAT16 is not "16" at all, as it uses a 14 bit address
IF %Totsectors% gtr 65536 SET PartTypes=06 07 0B 0C 0E
::FAT16 (06) has a limit in 2,147,483,648 bytes, i.e. 65,536 x 32 Kb clusters (64 sectors), i.e. 4,194,304
::Though it is possible, on NT based systems to have 64 Kb clusters (128 sectors), this feature is
:: rarely used, as the amount of wasted space is impressive and is not supported in this utility
::However the 06 type of partition, being CHS mapped has however a limit in the 1024th cylinder,
:: which is in common with FAT32 (0B) which is also CHS mapped, this limit in the "higher" possible
:: geometry of 1,024x255x63 allows to address 16,450,560 sectors, but on other geometries:
:: 1,024x128x63=8,257,536 
:: 1,024x64x32=2,097,152
:: 1,024x16x63=1,032,192
:: The 2K/XP FORMAT command will automatically change CHS mapped partitions to LBA mapped type
:: if they exceed the addressable range, so we need not worry about this
IF %Totsectors% gtr 4194304 SET PartTypes=07 0B 0C

IF NOT DEFINED Tempvar2 CALL :MKIMGI_PT
For %%A in (%PartTypes%) DO IF /I %Tempvar2%.==%%A. (SET PartType=%%A)
IF NOT DEFINED PartType CALL :MKIMGI_PT

For %%A in (01FAT 04FAT 06FAT 07NTFS 0BFAT32 0CFAT32 0EFAT) DO (
SET Tempvar=%%A
IF "!Tempvar:~0,2!".=="%PartType%". SET Filesystem=!Tempvar:~2,5!
)

IF NOT DEFINED Progname CALL :MKIMGI_PN
IF /I %Progname%.==/fsz. (
SET Progname=fsz.exe
SET Noprompts=%6
) ELSE (
SET Progname=mksparse.exe
SET Noprompts=%5
)

IF /I NOT %NoPrompts%.==/np. SET PDEL=/P



::We need to check if given filesize (without suffix) is actually a multiple of 512, otherwise mksparse.exe won't like it
:: if fsz.exe is used this does not matter, though VDK will only mount fully "even"  images, exceeding space will be ignored
IF %progname%==fsz.exe goto :Create_img
CALL :Do_Multiply %TotSectors% 512

IF NOT %Result%.==%Filesize%. ECHO Given size is not a multiple of 512!&GOTO :EOF



:Create_img

IF EXIST %Targetfile% del %PDEL% %Targetfile% 
%Progname% %Targetfile% %Filesize%

For /F  "tokens=1,2 delims=/" %%A in ("%Geometry%") DO Set MaxSect=%%B&SET MaxHead=%%A
SET /A MaxCyl=%Totsectors%/%MaxHead%/%MaxSect%
SET /A EndCyl=%MaxCyl%-1
SET /A EndHead=%MaxHead%-1
IF EXIST %Targetname%.$$$ del %PDEL% %Targetname%.$$$

CALL MBRBATCH.CMD CREATE %Targetname%.$$$
CALL MBRBATCH.CMD EDIT %Targetname%.$$$ %Geometry% CHS 1 %PartType% 80 0 1 1 %EndCyl% %EndHead% %MaxSect% /S
CALL :MAKEPLN %Targetfile%

:Check_Vdk_Status
FOR /F  %%A in ('VDK.EXE DRIVER^| FIND "NOT INSTALLED"') DO SET Status=OK
IF NOT %Status%.==OK. (
CLS
ECHO Batch has detected that an instance of VDK.EXE is already installed or running.
ECHO To avoid possible conflicts, you must stop and uninstall any instance of VDK,
ECHO before running MBRbatch.cmd, the batch will install and start it's own VDK.EXE.
ECHO.
GOTO :EOF
)
dsfi %Targetfile% 0 512 %Targetname%.$$$>nul
ECHO.
FOR /F  "tokens=7 delims= " %%A in ('ECHO I ^| VDK.EXE VIEW %Targetpln% ^| FIND /I "%PartType%h:"') DO (
ECHO Image will be mounted as %%A
SET ImageOK=%%A
)
IF NOT DEFINED ImageOK ECHO The image is unmountable, please check given parameters....&GOTO :EOF
VDK.EXE INSTALL >nul
VDK.EXE DISK 1 >nul
FOR /F  "tokens=1 delims= " %%A in ('ECHO I ^| VDK.EXE OPEN 0 %Targetpln% /RW^| FIND /I "%PartType%h:"') DO (
SET DRIVE=%%A
)
IF NOT DEFINED DRIVE GOTO :EOF
IF /I NOT %NoPrompts%.==/np. (
ECHO.
ECHO The batch has detected that the newly created image is mounted as drive %Drive%
ECHO.
ECHO NOW it would be the perfect time to stop if you are not sure about
ECHO what you are doing.
ECHO.
PAUSE
)

SET PFormat=FORMAT
IF /I %NoPrompts%.==/np. (
FOR /F "tokens=2,3 delims=(/)" %%A in ('ECHO ? ^| Format %Drive% /FS:%Filesystem% /Q /V:NO_NAME ^| FIND ")? ?"') DO SET YES=%%A
)
IF DEFINED YES SET PFormat=echo %YES%^| FORMAT

%Pformat% %Drive% /FS:%Filesystem% /Q /V:NO_NAME
IF /I NOT %NoPrompts%.==/np. (
START /WAIT %SystemRoot%\explorer.exe %drive%
ECHO.
ECHO Press any key to unmount the image and uninstall vdk.exe...
ECHO.
PAUSE
)
VDK.EXE REMOVE
GOTO :EOF

:MKIMGI_TF
SET /P Targetfile=Please enter target file name: 
IF NOT DEFINED Targetfile CLS &GOTO :MKIMGI_TF
CALL :get_Targetname %Targetfile%
GOTO :EOF
:MKIMGI_FS
ECHO Image size, in bytes or suffixed by K, M or G for Kilo Mega or Giga
SET /P Filesize=Please enter target image size: 
CALL :check_filesize
IF NOT DEFINED Filesize CLS &GOTO :MKIMGI_FS
GOTO :EOF

:MKIMGI_G
SET /P Tempvar1=Please type desired geometry [%Geometries%]: 
FOR %%A in (%Geometries%) DO IF %Tempvar1%.==%%A. (SET Geometry=%%A)
IF NOT DEFINED Geometry CLS &GOTO :MKIMGI_G
GOTO :EOF

:MKIMGI_PT
REM IF NOT DEFINED PartTypes SET PartTypes=01 04 06 07 0B 0C 0E
ECHO Available partition types for this image, %filesize% bytes:
FOR %%A IN (%PartTypes%) DO (
IF %%A==01 ECHO 01 FAT 12
IF %%A==04 ECHO 04 FAT 16 ^<32 Mbytes
IF %%A==06 ECHO 06 FAT 16 CHS Mapped
IF %%A==07 ECHO 07 NTFS
IF %%A==0B ECHO 0B FAT 32 CHS Mapped
IF %%A==0C ECHO 0C FAT 32 LBA Mapped
IF %%A==0E ECHO 0E FAT 16 LBA Mapped
)
SET /P Tempvar2=Please type desired Partition Type [%PartTypes%]: 
For %%A in (%PartTypes%) DO IF /I %Tempvar2%.==%%A. (SET PartType=%%A)
IF NOT DEFINED PartType CLS &GOTO :MKIMGI_PT
GOTO :EOF

:MKIMGI_PN
SET Progname=mksparse.exe
SET /P Progname=Please type /fsz to use fsz.exe or [ENTER] to use mksparse.exe: 
IF NOT %Progname%.==mksparse.exe. IF /I NOT %Progname%.==/fsz. SET Progname=
IF NOT DEFINED Progname CLS &GOTO :MKIMGI_PN
GOTO :EOF

:MAKEPLN
SET Targetpln=%~n1.pln
(
ECHO DRIVETYPE ide
ECHO CYLINDERS %MaxCyl%
ECHO HEADS %MaxHead%
ECHO SECTORS %MaxSect%
ECHO ACCESS "%~nx1" 0 %Totsectors%
)>%Targetpln%

GOTO :EOF

:getsize
SET /A Filesize=%~z1
GOTO :EOF

:Do_divide
Set Number=%1
Set Divider=%2
Set Separator=%1%2
For %%A in (D_Length N_Length %M_s% %N_s% %R_s% Result ) DO SET %%A=

::Find Separator
FOR %%A IN (0 1 2 3 4 5 6 7 8 9) DO IF DEFINED Separator Set Separator=!Separator:%%A=!
IF DEFINED Separator SET Separator=%Separator:~0,1%

::Remove dots or commas
IF DEFINED Separator Set Number=!Number:%Separator%=!
IF DEFINED Separator Set Divider=!Divider:%Separator%=!

FOR %%A IN (1 2 3 4 5 6 7 8 9 10 11 12 13 14 15) DO (
IF NOT DEFINED N_Length IF "!Number:~0,%%A!"=="!Number:~0!" SET N_Length=%%A
IF NOT DEFINED D_Length IF "!Divider:~0,%%A!"=="!Divider:~0!" SET D_Length=%%A
)

IF %D_Length% gtr 9 SET Result=0 &GOTO :EOF
SET /A Counter=0
SET divideArg=
CALL :divideloop
IF DEFINED Separator SET Digits=3&CALL :Putdots
GOTO :EOF

:divideloop
SET divideARG=%divideArg%!Number:~%Counter%,1!
SET /A Counter=%Counter%+1
SET /A Quotient=%divideArg%/%divider%
SET Result=%Result%%quotient%
IF %Result:~0,1%==0 SET Result=%Result:~1%
SET /A Rest=%divideArg% %% %divider%
IF %Rest%==0 SET Rest=
SET divideARG=%Rest%
IF %Counter% lss %N_length% GOTO :divideloop
GOTO :EOF

:Do_Multiply
Set Number=%1
Set Multiplier=%2
Set Separator=%1%2
For %%A in (MLength NLength %M_s% %N_s% %R_s% Result ) DO SET %%A=


::Find Separator
FOR %%A IN (0 1 2 3 4 5 6 7 8 9) DO IF DEFINED Separator Set Separator=!Separator:%%A=!
IF DEFINED Separator SET Separator=%Separator:~0,1%

::Remove dots or commas
IF DEFINED Separator Set Number=!Number:%Separator%=!
IF DEFINED Separator Set Multiplier=!Multiplier:%Separator%=!

FOR %%A IN (1 2 3 4 5 6 7 8 9 10 11 12 13 14 15) DO (
IF NOT DEFINED NLength IF "!Number:~0,%%A!"=="!Number:~0!" SET NLength=%%A
IF NOT DEFINED MLength IF "!Multiplier:~0,%%A!"=="!Multiplier:~0!" SET MLength=%%A
)

SET /A RLength=%NLength%+%MLength%
IF %Rlength% gtr 15 SET Result=0 &GOTO :EOF
Set Zeroes=000000000000000
SET N_s=N1
SET M_s=M1
SET R_s=R1
FOR %%A IN (1 2 3 4 5 6 7 8 9 10 11 12 13 14 15) DO (
IF %%A lss %NLength% SET N_s=!N_s! N1.!Zeroes:~0,%%A!
IF %%A lss %MLength% SET M_s=!M_s! M1.!Zeroes:~0,%%A!
IF %%A lss %RLength% SET R_s=!R_s! R1.!Zeroes:~0,%%A!
)


::Reverse numbers
SET Number=%Number:~14,1%%Number:~13,1%%Number:~12,1%%Number:~11,1%%Number:~10,1%%Number:~9,1%%Number:~8,1%%Number:~7,1%%Number:~6,1%%Number:~5,1%%Number:~4,1%%Number:~3,1%%Number:~2,1%%Number:~1,1%%Number:~0,1%
SET Multiplier=%Multiplier:~14,1%%Multiplier:~13,1%%Multiplier:~12,1%%Multiplier:~11,1%%Multiplier:~10,1%%Multiplier:~9,1%%Multiplier:~8,1%%Multiplier:~7,1%%Multiplier:~6,1%%Multiplier:~5,1%%Multiplier:~4,1%%Multiplier:~3,1%%Multiplier:~2,1%%Multiplier:~1,1%%Multiplier:~0,1%
Set Counter=0
FOR %%A IN (%N_s%) DO CALL :Assign_digits %%A Number
Set Counter=0
FOR %%A IN (%M_s%) DO CALL :Assign_digits %%A Multiplier


For %%A in (%M_s%) DO (
For %%B IN (%N_s%) DO (
IF DEFINED %%B CALL :Multiply %%A %%B
)
)
For %%A in (%R_s%) DO CALL :ReADD %%A
For %%A in (%R_s%) DO SET Result=!Result!!%%A!

::Reverse numbers
SET Result=%Result:~14,1%%Result:~13,1%%Result:~12,1%%Result:~11,1%%Result:~10,1%%Result:~9,1%%Result:~8,1%%Result:~7,1%%Result:~6,1%%Result:~5,1%%Result:~4,1%%Result:~3,1%%Result:~2,1%%Result:~1,1%%Result:~0,1%
IF %Result:~0,1%==0 SET Result=%Result:~1%
IF DEFINED Separator SET Digits=3&CALL :Putdots
GOTO :EOF

:Multiply
SET CurrR=R1%~x1%~x2
SET CurrR=%CurrR:0.=0%
SET CurrN=%1
SET CurrM=%2
SET /A %CurrR%=!%CurrR%!+!%CurrN%!*!%CurrM%!
GOTO :EOF

:ReADD
SET CurrR=%1
IF %CurrR%==R1 (SET CurrR=R1&SET NextR=R1.0) ELSE (SET NextR=%10)
IF NOT DEFINED %NextR% SET %NextR%=0
IF !%CurrR%! gtr 99 (
SET /A %NextR%=!%CurrR%:~-3,2!+!%NextR%!
SET /A %CurrR%=!%CurrR%:~-1,1!
)
IF !%CurrR%! gtr 9 (
SET /A %NextR%=!%CurrR%:~-2,1!+!%NextR%!
SET /A %CurrR%=!%CurrR%:~-1,1!
)
GOTO :EOF

:Putdots
Set Result=!Result:~0,-%Digits%!#!Result:~-%Digits%,%Digits%! 
Set /A Digits=%Digits%+5
IF NOT %Result:~0,1%==# GOTO :Putdots
Set Result=%Result:~1%
Set Result=!Result:#=%Separator%!
GOTO :EOF

:Assign_digits
SET %1=!%2:~%Counter%,1!
Set /A Counter=%Counter%+1
GOTO :EOF

:error1
ECHO Please add the missing file %1 in the same directory where %~nx0 is
GOTO :EOF

:get_Targetname
SET Targetname=%~n1
GOTO :EOF

:check_filesize
SET Filesize=%Filesize:,=%
SET Filesize=%Filesize:.=%
FOR %%A in ( K M G) do (
IF /I %Filesize:~-1,1%.==%%A. (
SET Suffix=%%A
SET Filesize=%Filesize:~0,-1%
)
)

SET is_number=%Filesize%
FOR %%A in (0 1 2 3 4 5 6 7 8 9) DO (
SET is_number=!is_number:%%A=!
IF NOT DEFINED is_number GOTO :is_number 
)
:is_number
IF DEFINED is_number SET Filesize=&GOTO :BEGIN

Set UnitM=1
FOR %%A IN (1024.K 1048576.M 1073741824.G) DO (
IF  /I .%Suffix%==%%~xA SET UnitM=%%~nA
)
CALL :Do_Multiply %Filesize% %UnitM%
SET Filesize=%Result%
GOTO :EOF